import multiprocessing
from time import sleep

import pytest

from texnomagic.client import TexnoMagicClient
from texnomagic.server import serve

import commontest  # common testing code


@pytest.fixture(autouse=True, scope="session")
def start_server():
    p = multiprocessing.Process(
        target=serve,
        kwargs={'abcs': commontest.ABCS})
    p.start()
    # give the server some time to start
    sleep(0.2)
    yield
    p.terminate()


@pytest.fixture(scope="function")
def client():
    c = TexnoMagicClient()
    c.connect()
    yield c
    c.close()


def test_req_invalid_method(client):
    reply = client.request('KEKW')
    assert 'error' in reply
    error = reply['error']
    assert error == {
        'code': -32601,
        'message': 'Method not found',
        'data': 'KEKW',
    }


def test_req_spell(client):
    reply = client.request(
        'spell',
        ['big slow ice death fast homing bolt random'],
    )
    assert 'error' not in reply
    assert 'result' in reply
    result = reply['result']
    assert result == {
        'direction': 'random',
        'effect': ['ice', 'death'],
        'effect_mods': {'size': 2, 'speed': 0.5},
        'spell': 'bolt',
        'spell_mods': {'homing': 1, 'speed': 2},
    }


def test_req_recognize(client):
    reply = client.request(
        'recognize',
        {
            'abc': commontest.ABC.name,
            'curves': [[[1,1], [10,10], [100, 100]]],
        })
    assert isinstance(reply, dict)
    assert 'error' not in reply
    assert 'result' in reply
    result = reply['result']
    assert 'symbol' in result
    assert 'score' in result



from pathlib import Path

import pytest

from texnomagic.lang import TexnoMagicLanguage


BASE_PATH = Path(__file__).parents[2]


@pytest.fixture(scope="module")
def txm():
    """
    test global instance of TexnoMagicLanguage
    """
    return TexnoMagicLanguage()


def test_bolt(txm):
    spell = txm.parse("big big ice fire death fast homing bolt random")
    exp = {
        "spell": "bolt",
        "effect": [
            "ice",
            "fire",
            "death"
        ],
        "effect_mods": {
            "size": 4
        },
        "spell_mods": {
            "speed": 2,
            "homing": 1
        },
        "direction": "random"
    }
    assert spell == exp


def test_self(txm):
    spell = txm.parse("big small big earth life self")
    exp = {
        "spell": "self",
        "effect": [
            "earth",
            "life"
        ],
        "effect_mods": {
            "size": 2.0
        }
    }
    assert spell == exp


def test_shield(txm):
    spell = txm.parse("small fast death ice shield area")
    exp = {
        "spell": "shield",
        "shape": "area",
        "effect": [
            "death",
            "ice"
        ],
        "effect_mods": {
            "size": 0.5,
            "speed": 2
        }
    }
    assert spell == exp



import shutil

import pytest

from texnomagic.abc import TexnoMagicAlphabet
from texnomagic.model import TexnoMagicSymbolModel

import commontest  # common testing code


@pytest.fixture(scope="module")
def abc(tmp_path_factory):
    """
    use a shared temporary copy of built-in testing alphabet in all tests
    """
    tmp_path = tmp_path_factory.mktemp('tenxomagic_tests')
    abc_path = tmp_path / commontest.ABC.path.name
    shutil.copytree(commontest.ABC.path, abc_path)

    abc = TexnoMagicAlphabet(path=abc_path)
    abc.load()
    abc.train_models()
    return abc


@pytest.fixture(scope="module", params=range(commontest.N_SYMBOLS))
def symbol(abc, request):
    i = request.param
    return abc.symbols[i]


def test_symbol_model_io(symbol):
    assert symbol.model.gmm
    drawing = symbol.random_drawing()
    score_orig = symbol.model.score(drawing)
    assert score_orig > -100

    model = TexnoMagicSymbolModel(path=symbol.model.path)
    model.load()
    score_new = model.score(drawing)

    # this means model save/load probably works :)
    assert score_new == score_orig


def test_symbol_model_recognize(abc):
    r = abc.check()
    assert r.get('error', []) == []
    for warn in r.get('warn', []):
        print("WARNING: %s" % warn)



import commontest  # common testing code


def test_abcs_basic():
    abcs = commontest.ABCS
    assert len(abcs.abcs) == 1, "only one abc should be loaded"

    abc = abcs.get_abc_by_name("DEFINITELY NOT EXISTING")
    assert abc is None

    abc = abcs.get_abc_by_name(commontest.ABC.name)
    assert abc.name == commontest.ABC.name
